@@ -6,7 +6,7 @@ from .math_functions.prior import make_prior
 from .normative_model import NormativeModel
 from .regression_model.blr import BLR
 from .regression_model.hbr import HBR
-from .util.plotter import plot_centiles, plot_qq, plot_ridge
+from .util.plotter import plot_centiles, plot_qq, plot_ridge, plot_centiles_advanced
 from .util.runner import Runner
 from importlib.metadata import version
 
@@ -28,4 +28,5 @@ __all__ = [
     "plot_ridge",
     "load_fcon1000",
     "Runner",
+    "plot_centiles_advanced"
 ]
