@@ -128,7 +128,7 @@ class NormData(xr.Dataset):
         Output.print(
             Messages.DATASET_CREATED,
             name=name,
-            n_subjects=len(np.unique(self.subjects)),
+            n_subjects=len(np.unique(self.subject_ids)),
             n_observations=len(self.observations),
             n_covariates=len(self.covariates),
             n_response_vars=len(self.response_vars),
@@ -136,91 +136,6 @@ class NormData(xr.Dataset):
             batch_effects=be_str,
         )
 
-    # @classmethod
-    # def from_ndarrays_old(
-    #     cls,
-    #     name: str,
-    #     X: np.ndarray,
-    #     Y: np.ndarray,
-    #     batch_effects: np.ndarray | None = None,
-    #     subject_ids: np.ndarray | None = None,
-    #     attrs: Mapping[str, Any] | None = None,
-    # ) -> NormData:
-    #     """Create a NormData object from numpy arrays.
-
-    #     Parameters
-    #     ----------
-    #     name : str
-    #         The name of the dataset
-    #     X : np.ndarray
-    #         Covariate data of shape (n_samples, n_features)
-    #     y : np.ndarray
-    #         Response variable data of shape (n_samples, n_responses)
-    #     batch_effects : np.ndarray
-    #         Batch effect data of shape (n_samples, n_batch_effects)
-    #     attrs : Mapping[str, Any] | None, optional
-    #         Additional attributes for the dataset, by default None
-
-    #     Returns
-    #     -------
-    #     NormData
-    #         A new NormData instance containing the provided data
-
-    #     Notes
-    #     -----
-    #     Input arrays are automatically reshaped to 2D if they are 1D
-    #     """
-    #     data_vars = {}
-    #     coords = {}
-    #     attrs = attrs or {}
-    #     if subject_ids is not None:
-    #         attrs["real_ids"] = True  # type:ignore
-    #         data_vars["subjects"] = (["observations"], subject_ids)
-    #     else:
-    #         attrs["real_ids"] = False  # type: ignore
-    #         data_vars["subjects"] = (["observations"], list(np.arange(X.shape[0])))
-
-    #     coords["observations"] = list(np.arange(X.shape[0]))
-    #     lengths = []
-    #     if X is not None:
-    #         lengths.append(X.shape[0])
-    #         if X.ndim == 1:
-    #             X = X[:, None]
-    #         data_vars["X"] = (["observations", "covariates"], X)
-    #         if "covariates" in attrs:
-    #             if len(attrs["covariates"]) != X.shape[1]:
-    #                 raise ValueError("The number of covariate names must match the number of covariates")
-    #             coords["covariates"] = attrs["covariates"]
-    #         else:
-    #             coords["covariates"] = [f"covariate_{i}" for i in np.arange(X.shape[1])]
-    #     if Y is not None:
-    #         lengths.append(Y.shape[0])
-    #         if Y.ndim == 1:
-    #             Y = Y[:, None]
-    #         data_vars["Y"] = (["observations", "response_vars"], Y)
-    #         if "response_vars" in attrs:
-    #             if len(attrs["response_vars"]) != Y.shape[1]:
-    #                 raise ValueError("The number of response names must match the number of response variables")
-    #             coords["response_vars"] = attrs["response_vars"]
-    #         else:
-    #             coords["response_vars"] = [f"response_var_{i}" for i in np.arange(Y.shape[1])]
-    #     if batch_effects is not None:
-    #         lengths.append(batch_effects.shape[0])
-    #         if batch_effects.ndim == 1:
-    #             batch_effects = batch_effects[:, None]
-    #         data_vars["batch_effects"] = (["observations", "batch_effect_dims"], batch_effects)
-    #         if "batch_effect_dims" in attrs:
-    #             if len(attrs["batch_effect_dims"]) != batch_effects.shape[1]:
-    #                 raise ValueError("The number of batch effect names must match the number of batch effects")
-    #             coords["batch_effect_dims"] = attrs["batch_effect_dims"]
-    #         else:
-    #             coords["batch_effect_dims"] = [f"batch_effect_{i}" for i in range(batch_effects.shape[1])]
-    #     else:
-    #         data_vars["batch_effects"] = (["observations", "batch_effect_dims"], np.zeros((lengths[0], 1)))
-    #         coords["batch_effect_dims"] = ["dummy_batch_effect"]
-    #     assert len(set(lengths)) == 1, "All arrays must have the same number of observations"
-    #     return cls(name, data_vars, coords, attrs)
-
     @classmethod
     def from_ndarrays(
         cls,
@@ -252,7 +167,7 @@ class NormData(xr.Dataset):
                     df_data[dataname] = array[:, i]
 
         if subject_ids is not None:
-            df_data["subjects"] = subject_ids
+            df_data["subject_ids"] = subject_ids
 
         return cls.from_dataframe(
             name=name,
@@ -263,7 +178,7 @@ class NormData(xr.Dataset):
                 [f"batch_effect_{i}" for i in range(batch_effects.shape[1])] if batch_effects is not None else None,
             ),
             response_vars=attrs.get("response_vars", [f"response_var_{i}" for i in range(Y.shape[1])] if Y is not None else None),
-            subject_ids="subjects" if subject_ids is not None else None,
+            subject_ids="subject_ids" if subject_ids is not None else None,
             attrs=attrs,
             remove_outliers=remove_outliers,
             z_threshold=z_threshold,
@@ -410,10 +325,10 @@ class NormData(xr.Dataset):
 
         if subject_ids is not None:
             attrs["real_ids"] = True  # type: ignore
-            data_vars["subjects"] = (["observations"], dataframe[subject_ids].to_numpy())
+            data_vars["subject_ids"] = (["observations"], dataframe[subject_ids].to_numpy())
         else:
             attrs["real_ids"] = False  # type: ignore
-            data_vars["subjects"] = (["observations"], list(np.arange(len(dataframe))))
+            data_vars["subject_ids"] = (["observations"], list(np.arange(len(dataframe))))
 
         coords["observations"] = list(np.arange(len(dataframe)))
 
@@ -452,6 +367,7 @@ class NormData(xr.Dataset):
         Output.print(f"Removed {len(dataframe) - len(cleaned)} NANs")
         return cleaned
 
+        
     @classmethod
     def remove_outliers(cls, dataframe: pd.DataFrame, continuous_vars: List[str], z_threshold: float = 3.0) -> pd.DataFrame:
         """
@@ -462,11 +378,15 @@ class NormData(xr.Dataset):
         idx = np.full(len(dataframe), True)
         for covar in continuous_vars:
             zscores = stats.zscore(dataframe[covar])
-            idx = idx & (np.abs(zscores) < z_threshold)
+            outliers = np.abs(zscores) > z_threshold
+            if sum(outliers) > 0:
+                Output.print(f"Removed {sum(outliers)} outliers for {covar}")
+            idx = idx & (~outliers)
         Output.print(f"Removed {np.sum(~idx)} outliers")
         return dataframe.loc[idx]
 
-    def merge(self, other: NormData) -> NormData:
+
+    def merge(self, other: NormData, name: str | None = None) -> NormData:
         """
         Merge two NormData objects.
 
@@ -477,12 +397,12 @@ class NormData(xr.Dataset):
         new_coords = {}
 
         if self.attrs["real_ids"] or other.attrs["real_ids"]:
-            new_data_vars["subjects"] = (
+            new_data_vars["subject_ids"] = (
                 ["observations"],
-                list(np.concatenate([self.subjects.to_numpy(), other.subjects.to_numpy()])),
+                list(np.concatenate([self.subject_ids.to_numpy(), other.subject_ids.to_numpy()])),
             )
         else:
-            new_data_vars["subjects"] = (["observations"], list(np.arange(self.X.shape[0] + other.X.shape[0])))
+            new_data_vars["subject_ids"] = (["observations"], list(np.arange(self.X.shape[0] + other.X.shape[0])))
 
         new_coords["observations"] = list(np.arange(self.X.shape[0] + other.X.shape[0]))
         covar_intersection = [c for c in self.covariates.to_numpy() if c in other.covariates.to_numpy()]
@@ -581,7 +501,7 @@ class NormData(xr.Dataset):
             new_data_vars["batch_effects"] = (["observations", "batch_effect_dims"], new_batch_effects.data)
 
         new_normdata = NormData(
-            name=self.attrs["name"],
+            name=name or (self.attrs["name"] + "_+_" + other.attrs["name"]),
             data_vars=new_data_vars,
             coords=new_coords,
             attrs=self.attrs,
@@ -732,6 +652,8 @@ class NormData(xr.Dataset):
         self.attrs["unique_batch_effects"] = {}
         self.attrs["batch_effect_counts"] = defaultdict(lambda: 0)
         self.attrs["covariate_ranges"] = {}
+
+        # TODO: the following can be done much easier using df.groupby.min and xarray.unstack, but that is a TODO for another day. This works for now.
         self.attrs["batch_effect_covariate_ranges"] = {}
         for dim in self.batch_effect_dims.to_numpy():
             dim_subset = my_be.sel(batch_effect_dims=dim)
@@ -748,12 +670,14 @@ class NormData(xr.Dataset):
                         my_c = self.X.sel(covariates=c).values[u_mask]
                         my_min = my_c.min()
                         my_max = my_c.max()
-                        self.attrs["batch_effect_covariate_ranges"][dim][u][c] = {"min": my_min, "max": my_max}
+                        my_mean = my_c.mean()
+                        self.attrs["batch_effect_covariate_ranges"][dim][u][c] = {"mean": my_mean, "min": my_min, "max": my_max}
         for c in self.covariates.to_numpy():
             my_c = self.X.sel(covariates=c).values
+            my_mean = my_c.mean()
             my_min = my_c.min()
             my_max = my_c.max()
-            self.attrs["covariate_ranges"][c] = {"min": my_min, "max": my_max}
+            self.attrs["covariate_ranges"][c] = {"mean": my_mean, "min": my_min, "max": my_max}
 
     def check_compatibility(self, other: NormData) -> bool:
         """
@@ -792,9 +716,11 @@ class NormData(xr.Dataset):
                 dataset_name=self.name,
                 response_vars=extra_response_vars,
             )
-        if len(missing_covariates) > 0 or len(extra_covariates) > 0 or len(extra_response_vars) > 0:
+
+        if len(missing_covariates) > 0 or len(extra_covariates) > 0:
             return False
-        return self.unique_batch_effects == other.unique_batch_effects
+
+        return self.unique_batch_effects.keys() == other.unique_batch_effects.keys()
 
     def make_compatible(self: NormData, other: NormData):
         """Ensures datasets are compatible by merging the batch effects maps"""
@@ -809,6 +735,7 @@ class NormData(xr.Dataset):
             for cov in self.covariates.to_numpy()
         }
 
+
         mybecr = self.batch_effect_covariate_ranges
         otbecr = other.batch_effect_covariate_ranges
         nbecr = {}
@@ -1084,11 +1011,11 @@ class NormData(xr.Dataset):
 
         acc.append(be)
 
-        subjects = xr.DataArray.to_dataframe(self.subjects, dim_order)[["subjects"]]
-        subjects.columns = [
-            ("subjects", "subjects"),
+        subject_ids = xr.DataArray.to_dataframe(self.subject_ids, dim_order)[["subject_ids"]]
+        subject_ids.columns = [
+            ("subject_ids", "subject_ids"),
         ]
-        acc.append(subjects)
+        acc.append(subject_ids)
         if hasattr(self, "centiles"):
             centiles = (
                 xr.DataArray.to_dataframe(self.centiles, dim_order)
@@ -1108,6 +1035,8 @@ class NormData(xr.Dataset):
     def save_zscores(self, save_dir: str) -> None:
         zdf = self.Z.to_dataframe().unstack(level="response_vars")
         zdf.columns = zdf.columns.droplevel(0)
+        zdf = zdf.merge(self.subject_ids.to_dataframe(), on="observations", how="left")
+        zdf = zdf[[ "subject_ids", *[z for z in sorted(zdf.columns.tolist()) if z not in ["subject_ids"]]]]
         zdf.index = zdf.index.astype(str)
         res_path = os.path.join(save_dir, f"Z_{self.name}.csv")
         with open(res_path, mode="a+" if os.path.exists(res_path) else "w", encoding="utf-8") as f:
@@ -1126,6 +1055,17 @@ class NormData(xr.Dataset):
                     new_results = zdf
                 f.seek(0)
                 f.truncate()
+                if "observations" in new_results.columns:
+                    new_results = new_results.sort_values(
+                        by="observations",
+                        key=lambda col: pd.to_numeric(col, errors="coerce"),
+                    )
+                    new_results["observations"] = new_results["observations"].astype(str)
+                else:
+                    new_results = new_results.sort_index(
+                        key=lambda idx: pd.to_numeric(idx, errors="coerce")
+                    )
+                    new_results.index = new_results.index.astype(str)
                 new_results.to_csv(f)
             finally:
                 fcntl.flock(f.fileno(), fcntl.LOCK_UN)
@@ -1145,6 +1085,15 @@ class NormData(xr.Dataset):
         centiles = self.centiles.to_dataframe().unstack(level="response_vars")
         centiles.columns = centiles.columns.droplevel(0)
         centiles.index = centiles.index.set_levels(centiles.index.levels[1].astype(str), level=1)
+        subject_ids = self.subject_ids.to_dataframe()
+        subject_ids.index = subject_ids.index.astype(str)
+        subject_ids.columns = pd.MultiIndex.from_tuples([("subject_ids", "X")], names=["subject_ids", "centile"])
+        for c in self.centile.to_numpy():
+            subject_ids[("subject_ids", c)] = subject_ids[("subject_ids","X")]
+        subject_ids = subject_ids.drop(columns=[("subject_ids", "X")])
+        subject_ids = subject_ids.stack(level="centile")
+        centiles = centiles.merge(subject_ids, on=["observations","centile"], how="left")
+        centiles = centiles[[ "subject_ids", *[z for z in sorted(centiles.columns.tolist()) if z not in ["subject_ids"]]]]
         res_path = os.path.join(save_dir, f"centiles_{self.name}.csv")
         with open(res_path, mode="a+" if os.path.exists(res_path) else "w", encoding="utf-8") as f:
             try:
@@ -1162,6 +1111,17 @@ class NormData(xr.Dataset):
                     new_results = centiles
                 f.seek(0)
                 f.truncate()
+                if "observations" in new_results.columns:
+                    new_results = new_results.sort_values(
+                        by="observations",
+                        key=lambda col: pd.to_numeric(col, errors="coerce"),
+                    )
+                    new_results["observations"] = new_results["observations"].astype(str)
+                else:
+                    new_results = new_results.sort_index(
+                        key=lambda idx: pd.to_numeric(idx, errors="coerce")
+                    )
+                    # new_results.index = new_results.index.astype(str)
                 new_results.to_csv(f)
             finally:
                 fcntl.flock(f.fileno(), fcntl.LOCK_UN)
@@ -1170,7 +1130,7 @@ class NormData(xr.Dataset):
         C_path = os.path.join(save_dir, f"centiles_{self.name}.csv")
         if os.path.isfile(C_path):
             df = pd.read_csv(C_path)
-            response_vars = [i for i in list(df.columns) if not (i == "observations" or i == "centile")]
+            response_vars = [i for i in list(df.columns) if not (i == "observations" or i == "centile" or i == "subject_ids")]
             centiles = np.unique(df["centile"])
             obs = np.unique(df["observations"])
             obs.sort()
@@ -1190,6 +1150,8 @@ class NormData(xr.Dataset):
     def save_logp(self, save_dir: str) -> None:
         logp = self.logp.to_dataframe().unstack(level="response_vars")
         logp.columns = logp.columns.droplevel(0)
+        logp = logp.merge(self.subject_ids.to_dataframe(), on="observations", how="left")
+        logp = logp[[ "subject_ids", *[z for z in sorted(logp.columns.tolist()) if z not in ["subject_ids"]]]]
         logp.index = logp.index.astype(str)
         res_path = os.path.join(save_dir, f"logp_{self.name}.csv")
         with open(res_path, mode="a+" if os.path.exists(res_path) else "w", encoding="utf-8") as f:
@@ -1208,6 +1170,17 @@ class NormData(xr.Dataset):
                     new_results = logp
                 f.seek(0)
                 f.truncate()
+                if "observations" in new_results.columns:
+                    new_results = new_results.sort_values(
+                        by="observations",
+                        key=lambda col: pd.to_numeric(col, errors="coerce"),
+                    )
+                    new_results["observations"] = new_results["observations"].astype(str)
+                else:
+                    new_results = new_results.sort_index(
+                        key=lambda idx: pd.to_numeric(idx, errors="coerce")
+                    )
+                    new_results.index = new_results.index.astype(str)
                 new_results.to_csv(f)
             finally:
                 fcntl.flock(f.fileno(), fcntl.LOCK_UN)
@@ -1308,7 +1281,6 @@ class NormData(xr.Dataset):
             self.statistics.to_dataframe()
             .reset_index()
             .pivot(index=["response_vars"], columns=["statistic"], values="statistics")
-            .round(2)
         )
         return statistics_df
 
